@@ -1089,7 +1089,9 @@ typedef enum {
     XML_PARSE_NSCLEAN	= 1<<13,/* remove redundant namespaces declarations */
     XML_PARSE_NOCDATA	= 1<<14,/* merge CDATA as text nodes */
     XML_PARSE_NOXINCNODE= 1<<15,/* do not generate XINCLUDE START/END nodes */
-    XML_PARSE_COMPACT   = 1<<16 /* compact small text nodes */
+    XML_PARSE_COMPACT   = 1<<16 /* compact small text nodes; no modification of
+                                   the tree allowed afterwards (will possibly
+				   crash if you try to modify the tree) */
 } xmlParserOption;
 
 XMLPUBFUN void XMLCALL
@@ -1171,39 +1173,41 @@ XMLPUBFUN xmlDocPtr XMLCALL
  *
  * Used to examine the existance of features that can be enabled
  * or disabled at compile-time.
+ * They used to be called XML_FEATURE_xxx but this clashed with Expat
  */
 typedef enum {
-    XML_FEATURE_THREAD = 1,
-    XML_FEATURE_TREE = 2,
-    XML_FEATURE_OUTPUT = 3,
-    XML_FEATURE_PUSH = 4,
-    XML_FEATURE_READER = 5,
-    XML_FEATURE_PATTERN = 6,
-    XML_FEATURE_WRITER = 7,
-    XML_FEATURE_SAX1 = 8,
-    XML_FEATURE_FTP = 9,
-    XML_FEATURE_HTTP = 10,
-    XML_FEATURE_VALID = 11,
-    XML_FEATURE_HTML = 12,
-    XML_FEATURE_LEGACY = 13,
-    XML_FEATURE_C14N = 14,
-    XML_FEATURE_CATALOG = 15,
-    XML_FEATURE_XPATH = 16,
-    XML_FEATURE_XPTR = 17,
-    XML_FEATURE_XINCLUDE = 18,
-    XML_FEATURE_ICONV = 19,
-    XML_FEATURE_ISO8859X = 20,
-    XML_FEATURE_UNICODE = 21,
-    XML_FEATURE_REGEXP = 22,
-    XML_FEATURE_AUTOMATA = 23,
-    XML_FEATURE_EXPR = 24,
-    XML_FEATURE_SCHEMAS = 25,
-    XML_FEATURE_SCHEMATRON = 26,
-    XML_FEATURE_MODULES = 27,
-    XML_FEATURE_DEBUG = 28,
-    XML_FEATURE_DEBUG_MEM = 29,
-    XML_FEATURE_DEBUG_RUN = 30,
-    XML_FEATURE_NONE = 99999 /* just to be sure of allocation size */
+    XML_WITH_THREAD = 1,
+    XML_WITH_TREE = 2,
+    XML_WITH_OUTPUT = 3,
+    XML_WITH_PUSH = 4,
+    XML_WITH_READER = 5,
+    XML_WITH_PATTERN = 6,
+    XML_WITH_WRITER = 7,
+    XML_WITH_SAX1 = 8,
+    XML_WITH_FTP = 9,
+    XML_WITH_HTTP = 10,
+    XML_WITH_VALID = 11,
+    XML_WITH_HTML = 12,
+    XML_WITH_LEGACY = 13,
+    XML_WITH_C14N = 14,
+    XML_WITH_CATALOG = 15,
+    XML_WITH_XPATH = 16,
+    XML_WITH_XPTR = 17,
+    XML_WITH_XINCLUDE = 18,
+    XML_WITH_ICONV = 19,
+    XML_WITH_ISO8859X = 20,
+    XML_WITH_UNICODE = 21,
+    XML_WITH_REGEXP = 22,
+    XML_WITH_AUTOMATA = 23,
+    XML_WITH_EXPR = 24,
+    XML_WITH_SCHEMAS = 25,
+    XML_WITH_SCHEMATRON = 26,
+    XML_WITH_MODULES = 27,
+    XML_WITH_DEBUG = 28,
+    XML_WITH_DEBUG_MEM = 29,
+    XML_WITH_DEBUG_RUN = 30,
+    XML_WITH_ZLIB = 31,
+    XML_WITH_NONE = 99999 /* just to be sure of allocation size */
 } xmlFeature;
 
 XMLPUBFUN int XMLCALL
