@@ -98,6 +98,9 @@
 #ifdef LIBXML_C14N_ENABLED
 #include <libxml/c14n.h>
 #endif
+#ifdef LIBXML_OUTPUT_ENABLED
+#include <libxml/xmlsave.h>
+#endif
 
 #ifndef XML_XML_DEFAULT_CATALOG
 #define XML_XML_DEFAULT_CATALOG "file:///etc/xml/catalog"
@@ -133,6 +136,7 @@ static int nowrap = 0;
 static int format = 0;
 static const char *output = NULL;
 static int compress = 0;
+static int oldout = 0;
 #endif /* LIBXML_OUTPUT_ENABLED */
 #ifdef LIBXML_VALID_ENABLED
 static int valid = 0;
@@ -146,7 +150,7 @@ static xmlRelaxNGPtr relaxngschemas = NULL;
 static char * schema = NULL;
 static xmlSchemaPtr wxschemas = NULL;
 #endif
-#ifdef LIBXML_SCHEMAS_ENABLED
+#ifdef LIBXML_SCHEMATRON_ENABLED
 static char * schematron = NULL;
 static xmlSchematronPtr wxschematron = NULL;
 #endif
@@ -205,6 +209,11 @@ static int sax = 0;
  *									*
  ************************************************************************/
 #define MAX_PATHS 64
+#ifdef _WIN32
+# define PATH_SEPARATOR ';'
+#else
+# define PATH_SEPARATOR ':'
+#endif
 static xmlChar *paths[MAX_PATHS + 1];
 static int nbpaths = 0;
 static int load_trace = 0;
@@ -221,10 +230,10 @@ void parsePath(const xmlChar *path) {
 	    return;
 	}
 	cur = path;
-	while ((*cur == ' ') || (*cur == ':'))
+	while ((*cur == ' ') || (*cur == PATH_SEPARATOR))
 	    cur++;
 	path = cur;
-	while ((*cur != 0) && (*cur != ' ') && (*cur != ':'))
+	while ((*cur != 0) && (*cur != ' ') && (*cur != PATH_SEPARATOR))
 	    cur++;
 	if (cur != path) {
 	    paths[nbpaths] = xmlStrndup(path, cur - path);
@@ -242,6 +251,7 @@ xmllintExternalEntityLoader(const char *URL, const char *ID,
 			     xmlParserCtxtPtr ctxt) {
     xmlParserInputPtr ret;
     warningSAXFunc warning = NULL;
+    errorSAXFunc err = NULL;
 
     int i;
     const char *lastsegment = URL;
@@ -257,7 +267,9 @@ xmllintExternalEntityLoader(const char *URL, const char *ID,
 
     if ((ctxt != NULL) && (ctxt->sax != NULL)) {
 	warning = ctxt->sax->warning;
+	err = ctxt->sax->error;
 	ctxt->sax->warning = NULL;
+	ctxt->sax->error = NULL;
     }
 
     if (defaultEntityLoader != NULL) {
@@ -265,6 +277,8 @@ xmllintExternalEntityLoader(const char *URL, const char *ID,
 	if (ret != NULL) {
 	    if (warning != NULL)
 		ctxt->sax->warning = warning;
+	    if (err != NULL)
+		ctxt->sax->error = err;
 	    if (load_trace) {
 		fprintf \
 			(stderr,
@@ -286,6 +300,8 @@ xmllintExternalEntityLoader(const char *URL, const char *ID,
 	    if (ret != NULL) {
 		if (warning != NULL)
 		    ctxt->sax->warning = warning;
+		if (err != NULL)
+		    ctxt->sax->error = err;
 		if (load_trace) {
 		    fprintf \
 		    	(stderr,
@@ -299,6 +315,8 @@ xmllintExternalEntityLoader(const char *URL, const char *ID,
 	    xmlFree(newURL);
 	}
     }
+    if (err != NULL)
+        ctxt->sax->error = err;
     if (warning != NULL) {
 	ctxt->sax->warning = warning;
 	if (URL != NULL)
@@ -2080,6 +2098,26 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
         }
     }
 #endif /* LIBXML_PUSH_ENABLED */
+#ifdef HAVE_SYS_MMAN_H
+    else if ((html) && (memory)) {
+	int fd;
+	struct stat info;
+	const char *base;
+	if (stat(filename, &info) < 0) 
+	    return;
+	if ((fd = open(filename, O_RDONLY)) < 0)
+	    return;
+	base = mmap(NULL, info.st_size, PROT_READ, MAP_SHARED, fd, 0) ;
+	if (base == (void *) MAP_FAILED)
+	    return;
+
+	doc = htmlReadMemory((char *) base, info.st_size, filename,
+	                     NULL, options);
+	    
+	munmap((char *) base, info.st_size);
+	close(fd);
+    }
+#endif
     else if (html) {
 	doc = htmlReadFile(filename, NULL, options);
     }
@@ -2194,6 +2232,7 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 			                filename, NULL, options);
 	        
 	    munmap((char *) base, info.st_size);
+	    close(fd);
 #endif
 #ifdef LIBXML_VALID_ENABLED
 	} else if (valid) {
@@ -2271,8 +2310,10 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
     /*
      * shell interaction
      */
-    if (shell)  
+    if (shell) {
+        xmlXPathOrderDocElems(doc);
         xmlShell(doc, filename, xmlShellReadline, stdout);
+    }
 #endif
 #endif
 
@@ -2434,48 +2475,71 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 		    write(1, result, len);
 		    xmlFree(result);
 		}
+
 	    } else
 #endif /* HAVE_SYS_MMAN_H */
 	    if (compress) {
 		xmlSaveFile(output ? output : "-", doc);
-	    }
-	    else if (encoding != NULL) {
-	        if ( format ) {
-		    ret = xmlSaveFormatFileEnc(output ? output : "-", doc,
-		                               encoding, 1);
+	    } else if (oldout) {
+	        if (encoding != NULL) {
+		    if ( format ) {
+			ret = xmlSaveFormatFileEnc(output ? output : "-", doc,
+						   encoding, 1);
+		    }
+		    else {
+			ret = xmlSaveFileEnc(output ? output : "-", doc,
+			                     encoding);
+		    }
+		    if (ret < 0) {
+			fprintf(stderr, "failed save to %s\n",
+				output ? output : "-");
+			progresult = XMLLINT_ERR_OUT;
+		    }
+		} else if (format) {
+		    ret = xmlSaveFormatFile(output ? output : "-", doc, 1);
+		    if (ret < 0) {
+			fprintf(stderr, "failed save to %s\n",
+				output ? output : "-");
+			progresult = XMLLINT_ERR_OUT;
+		    }
+		} else {
+		    FILE *out;
+		    if (output == NULL)
+			out = stdout;
+		    else {
+			out = fopen(output,"wb");
+		    }
+		    if (out != NULL) {
+			if (xmlDocDump(out, doc) < 0)
+			    progresult = XMLLINT_ERR_OUT;
+
+			if (output != NULL)
+			    fclose(out);
+		    } else {
+			fprintf(stderr, "failed to open %s\n", output);
+			progresult = XMLLINT_ERR_OUT;
+		    }
 		}
-		else {
-		    ret = xmlSaveFileEnc(output ? output : "-", doc, encoding);
-		}
-		if (ret < 0) {
-		    fprintf(stderr, "failed save to %s\n",
-		            output ? output : "-");
-		    progresult = XMLLINT_ERR_OUT;
-		}
-	    }
-	    else if (format) {
-		ret = xmlSaveFormatFile(output ? output : "-", doc, 1);
-		if (ret < 0) {
-		    fprintf(stderr, "failed save to %s\n",
-		            output ? output : "-");
-		    progresult = XMLLINT_ERR_OUT;
-		}
-	    }
-	    else {
-		FILE *out;
+	    } else {
+	        xmlSaveCtxtPtr ctxt;
+		int saveOpts = 0;
+
+                if (format)
+		    saveOpts |= XML_SAVE_FORMAT;
+
 		if (output == NULL)
-		    out = stdout;
-		else {
-		    out = fopen(output,"wb");
-		}
-		if (out != NULL) {
-		    if (xmlDocDump(out, doc) < 0)
-		        progresult = XMLLINT_ERR_OUT;
+		    ctxt = xmlSaveToFd(1, encoding, saveOpts);
+		else
+		    ctxt = xmlSaveToFilename(output, encoding, saveOpts);
 
-		    if (output != NULL)
-			fclose(out);
+		if (ctxt != NULL) {
+		    if (xmlSaveDoc(ctxt, doc) < 0) {
+			fprintf(stderr, "failed save to %s\n",
+				output ? output : "-");
+			progresult = XMLLINT_ERR_OUT;
+		    }
+		    xmlSaveClose(ctxt);
 		} else {
-		    fprintf(stderr, "failed to open %s\n", output);
 		    progresult = XMLLINT_ERR_OUT;
 		}
 	    }
@@ -2713,36 +2777,37 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 static void showVersion(const char *name) {
     fprintf(stderr, "%s: using libxml version %s\n", name, xmlParserVersion);
     fprintf(stderr, "   compiled with: ");
-    if (xmlHasFeature(XML_FEATURE_THREAD)) fprintf(stderr, "Threads ");
-    if (xmlHasFeature(XML_FEATURE_TREE)) fprintf(stderr, "Tree ");
-    if (xmlHasFeature(XML_FEATURE_OUTPUT)) fprintf(stderr, "Output ");
-    if (xmlHasFeature(XML_FEATURE_PUSH)) fprintf(stderr, "Push ");
-    if (xmlHasFeature(XML_FEATURE_READER)) fprintf(stderr, "Reader ");
-    if (xmlHasFeature(XML_FEATURE_PATTERN)) fprintf(stderr, "Patterns ");
-    if (xmlHasFeature(XML_FEATURE_WRITER)) fprintf(stderr, "Writer ");
-    if (xmlHasFeature(XML_FEATURE_SAX1)) fprintf(stderr, "SAXv1 ");
-    if (xmlHasFeature(XML_FEATURE_FTP)) fprintf(stderr, "FTP "); 
-    if (xmlHasFeature(XML_FEATURE_HTTP)) fprintf(stderr, "HTTP "); 
-    if (xmlHasFeature(XML_FEATURE_VALID)) fprintf(stderr, "DTDValid ");
-    if (xmlHasFeature(XML_FEATURE_HTML)) fprintf(stderr, "HTML "); 
-    if (xmlHasFeature(XML_FEATURE_LEGACY)) fprintf(stderr, "Legacy "); 
-    if (xmlHasFeature(XML_FEATURE_C14N)) fprintf(stderr, "C14N "); 
-    if (xmlHasFeature(XML_FEATURE_CATALOG)) fprintf(stderr, "Catalog "); 
-    if (xmlHasFeature(XML_FEATURE_XPATH)) fprintf(stderr, "XPath "); 
-    if (xmlHasFeature(XML_FEATURE_XPTR)) fprintf(stderr, "XPointer "); 
-    if (xmlHasFeature(XML_FEATURE_XINCLUDE)) fprintf(stderr, "XInclude "); 
-    if (xmlHasFeature(XML_FEATURE_ICONV)) fprintf(stderr, "Iconv "); 
-    if (xmlHasFeature(XML_FEATURE_ISO8859X)) fprintf(stderr, "ISO8859X "); 
-    if (xmlHasFeature(XML_FEATURE_UNICODE)) fprintf(stderr, "Unicode "); 
-    if (xmlHasFeature(XML_FEATURE_REGEXP)) fprintf(stderr, "Regexps "); 
-    if (xmlHasFeature(XML_FEATURE_AUTOMATA)) fprintf(stderr, "Automata "); 
-    if (xmlHasFeature(XML_FEATURE_EXPR)) fprintf(stderr, "Expr "); 
-    if (xmlHasFeature(XML_FEATURE_SCHEMAS)) fprintf(stderr, "Schemas "); 
-    if (xmlHasFeature(XML_FEATURE_SCHEMATRON)) fprintf(stderr, "Schematron "); 
-    if (xmlHasFeature(XML_FEATURE_MODULES)) fprintf(stderr, "Modules "); 
-    if (xmlHasFeature(XML_FEATURE_DEBUG)) fprintf(stderr, "Debug "); 
-    if (xmlHasFeature(XML_FEATURE_DEBUG_MEM)) fprintf(stderr, "MemDebug "); 
-    if (xmlHasFeature(XML_FEATURE_DEBUG_RUN)) fprintf(stderr, "RunDebug "); 
+    if (xmlHasFeature(XML_WITH_THREAD)) fprintf(stderr, "Threads ");
+    if (xmlHasFeature(XML_WITH_TREE)) fprintf(stderr, "Tree ");
+    if (xmlHasFeature(XML_WITH_OUTPUT)) fprintf(stderr, "Output ");
+    if (xmlHasFeature(XML_WITH_PUSH)) fprintf(stderr, "Push ");
+    if (xmlHasFeature(XML_WITH_READER)) fprintf(stderr, "Reader ");
+    if (xmlHasFeature(XML_WITH_PATTERN)) fprintf(stderr, "Patterns ");
+    if (xmlHasFeature(XML_WITH_WRITER)) fprintf(stderr, "Writer ");
+    if (xmlHasFeature(XML_WITH_SAX1)) fprintf(stderr, "SAXv1 ");
+    if (xmlHasFeature(XML_WITH_FTP)) fprintf(stderr, "FTP "); 
+    if (xmlHasFeature(XML_WITH_HTTP)) fprintf(stderr, "HTTP "); 
+    if (xmlHasFeature(XML_WITH_VALID)) fprintf(stderr, "DTDValid ");
+    if (xmlHasFeature(XML_WITH_HTML)) fprintf(stderr, "HTML "); 
+    if (xmlHasFeature(XML_WITH_LEGACY)) fprintf(stderr, "Legacy "); 
+    if (xmlHasFeature(XML_WITH_C14N)) fprintf(stderr, "C14N "); 
+    if (xmlHasFeature(XML_WITH_CATALOG)) fprintf(stderr, "Catalog "); 
+    if (xmlHasFeature(XML_WITH_XPATH)) fprintf(stderr, "XPath "); 
+    if (xmlHasFeature(XML_WITH_XPTR)) fprintf(stderr, "XPointer "); 
+    if (xmlHasFeature(XML_WITH_XINCLUDE)) fprintf(stderr, "XInclude "); 
+    if (xmlHasFeature(XML_WITH_ICONV)) fprintf(stderr, "Iconv "); 
+    if (xmlHasFeature(XML_WITH_ISO8859X)) fprintf(stderr, "ISO8859X "); 
+    if (xmlHasFeature(XML_WITH_UNICODE)) fprintf(stderr, "Unicode "); 
+    if (xmlHasFeature(XML_WITH_REGEXP)) fprintf(stderr, "Regexps "); 
+    if (xmlHasFeature(XML_WITH_AUTOMATA)) fprintf(stderr, "Automata "); 
+    if (xmlHasFeature(XML_WITH_EXPR)) fprintf(stderr, "Expr "); 
+    if (xmlHasFeature(XML_WITH_SCHEMAS)) fprintf(stderr, "Schemas "); 
+    if (xmlHasFeature(XML_WITH_SCHEMATRON)) fprintf(stderr, "Schematron "); 
+    if (xmlHasFeature(XML_WITH_MODULES)) fprintf(stderr, "Modules "); 
+    if (xmlHasFeature(XML_WITH_DEBUG)) fprintf(stderr, "Debug "); 
+    if (xmlHasFeature(XML_WITH_DEBUG_MEM)) fprintf(stderr, "MemDebug "); 
+    if (xmlHasFeature(XML_WITH_DEBUG_RUN)) fprintf(stderr, "RunDebug "); 
+    if (xmlHasFeature(XML_WITH_ZLIB)) fprintf(stderr, "Zlib ");
     fprintf(stderr, "\n");
 }
 
@@ -3155,6 +3220,7 @@ main(int argc, char **argv) {
         } else if ((!strcmp(argv[i], "-nonet")) ||
                    (!strcmp(argv[i], "--nonet"))) {
 	    options |= XML_PARSE_NONET;
+	    xmlSetExternalEntityLoader(xmlNoNetExternalEntityLoader);
         } else if ((!strcmp(argv[i], "-nocompact")) ||
                    (!strcmp(argv[i], "--nocompact"))) {
 	    options &= ~XML_PARSE_COMPACT;
